/*
 * Copyright:
 * ----------------------------------------------------------------
 * This confidential and proprietary software may be used only as
 * authorised by a licensing agreement from ARM Limited
 *   (C) COPYRIGHT 2013 ARM Limited
 *       ALL RIGHTS RESERVED
 * The entire notice above must be reproduced on all authorised
 * copies and copies may only be made to the extent permitted
 * by a licensing agreement from ARM Limited.
 * ----------------------------------------------------------------
 * File:     demo1.c
 * Release:  Version 1.0
 * ----------------------------------------------------------------
 */

/*
 * Code implementation file for the Color DEMO1.
 */

#include <stdio.h>
#include <stdlib.h>

#include "GLCD_SPI_MPS2.h"

#include "demo1.h"
#include "icons.h"

#include "../main/common.h"
#include "../aptsc/aptsc.h"

// Demo 1 Screen initialisation
void apDEMO1_init(void)
{
	// Set background to black
	GLCD_Clear (Black);

	// Draw EXIT button image
    GLCD_BitmapSize (268, 319, 185, 239, exitData);

    // Draw the three sliders
    GLCD_BitmapSize (60, 235, 110, 137, slideData);

    GLCD_BitmapSize (60, 235, 160, 187, slideData);

    GLCD_BitmapSize (60, 235, 210, 237, slideData);

    // Draw next icon
    GLCD_BitmapSize (275, 310, 119, 156, nextData);
}

// Demo 1 test
void apDEMO1_run(void)
{
	unsigned int done, ucol, ncar, carno;
	unsigned int rpos, gpos, bpos, pend, penx, peny, nx, ncol;
    unsigned short change_color[2];

    // Screen Initialisation
    apDEMO1_init();
    
	// Set default sliders
	done  = FALSE;
	ucol  = TRUE;
	ncar  = FALSE;
	carno = 0;
	rpos  = 64;
	gpos  = 64;
	bpos  = 64;
	penx  = 64 + 86;
	apTSC_setslide(rpos + 86, 112, Red, penx);
	apTSC_setslide(gpos + 86, 162, Green, penx);
	apTSC_setslide(bpos + 86, 212, Blue, penx);

	do
	{
		// Read touchscreen
		pend = apTSC_READXY(&penx, &peny);

		// Done
		if (pend && (penx > 238) && (peny > 200))
		    done = TRUE;

		// Red cursor
		if (pend && (penx > 76) && (penx < 224) && (peny > 95) && (peny < 135))
		{
			nx = (penx < 214) ? penx : 213;
			nx = (nx > 86) ? nx : 87;
			apTSC_setslide(rpos + 86, 112, Red, nx);
			rpos = nx - 86;
			ucol = TRUE;
		}
		// Green cursor
		if (pend && (penx > 76) && (penx < 224) && (peny > 147) && (peny < 185))
		{
			nx = (penx < 214) ? penx : 213;
			nx = (nx > 86) ? nx : 87;
			apTSC_setslide(gpos + 86, 162, Green, nx);
			gpos = nx - 86;
			ucol = TRUE;
		}
		// Blue cursor
		if (pend && (penx > 76) && (penx < 224) && (peny > 195) && (peny < 235))
		{
			nx = (penx < 214) ? penx : 213;
			nx = (nx > 86) ? nx : 87;
			apTSC_setslide(bpos + 86, 212, Blue, nx);
			bpos = nx - 86;
			ucol = TRUE;
		}

		// New car
		if (pend && (penx > 275) && (penx < 310) && (peny > 119) && (peny < 156))
		{
			// Clear the old car
            GLCD_BoxSize(0, 319, 0, 110, 0x0000);

			// Next car
			carno = (carno == 2) ? 0 : carno + 1;
			ncar = TRUE;
		}

	    // Draw the new car colour
		if (ucol || ncar)
		{
			// Calculate car colour
			ncol = ((rpos << 9) & Red) | ((gpos << 4) & Green) | ((bpos >> 2) & Blue);

			// Draw car, original base color is Red 
            change_color[0] = Red;
            change_color[1] = ncol;
            
			if (carno == 0)
			{
                GLCD_ChangeBitMap (28, 271, 5, 86, car1Data, 1, change_color);
			}
			else if (carno == 1)
			{
                GLCD_ChangeBitMap (36, 263, 0, 100, car2Data, 1, change_color);
			}
			else
			{
                GLCD_ChangeBitMap (26, 277, 0, 99, car3Data, 1, change_color);
			}
			if (ncar)
				apSleep(1000);

			ucol = FALSE;
			ncar = FALSE;
		}
	} while (!done);

	// Change EXIT image
    change_color[0] = White;
    change_color[1] = Green;
    GLCD_ChangeBitMap (268, 319, 185, 239, exitData, 1, change_color);

	apSleep(1000);
}
