/*
 * Copyright:
 * ----------------------------------------------------------------
 * This confidential and proprietary software may be used only as
 * authorised by a licensing agreement from ARM Limited
 *   (C) COPYRIGHT 2013 ARM Limited
 *       ALL RIGHTS RESERVED
 * The entire notice above must be reproduced on all authorised
 * copies and copies may only be made to the extent permitted
 * by a licensing agreement from ARM Limited.
 * ----------------------------------------------------------------
 * File:     main.c
 * Release:  Version 2.0
 * ----------------------------------------------------------------
 *
 */

/*
 * --------Included Headers--------
 */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdint.h>
#include <stdlib.h>
#include <time.h>

#include "SMM_MPS2.h"                   // MPS2 common header
#include "GLCD_SPI_MPS2.h"

#include "uart_stdout.h"
#include "common.h"
#include "icons.h"

#include "../aptsc/aptsc.h"
#include "../demo/demo1.h"
#include "../demo/demo2.h"
#include "../demo/demo3.h"

// Display Colour bars on the VGA screen
void VGA_Test (void)
{
    unsigned int x, y, pxl, col;

    for (y = 0; y < 128; y++)
    {
        for (x = 0; x < (128 * 3); x++)
        {
            // Colour 0x0000RGB (4 bit colour)
            if (x < 128)
                col = ((x >> 3) & 0xF);
            else if (x < 256)
                col = ((x >> 3) & 0xF) << 4;
            else
                col = ((x >> 3) & 0xF) << 8;

            // Pixel position
            pxl = (x * 4) + (y * 512 * 4);

            // Write to screen buffer
            *((unsigned int *)(MPS2_VGA_BUFFER + pxl)) = col;
        }
    }
}

unsigned int mainmenu (void)
{
    unsigned int state, pend, penx, peny, button, timer;

    GLCD_Clear (Black);
    
    // Fill display with test pattern bitmap (320*240*16bit RGB 5:6:5)
    GLCD_Bitmap (0, 0, 320, 240, (unsigned short *)flyerData);

	// Wait for button press
	state  = 0;
	button = 0;
	timer  = 0;
    do
    {
		// Draw PRESS screen image box (8*74)
    	if (!timer)
    	{
            if(!state)
                GLCD_BitmapSize (231, 238, 110, 184, (unsigned short *)pressData);
            else
                GLCD_BoxSize (231, 238, 110, 184, 0x0BF3);
            
			state = state ^ 0x01;
    	}

		// Read touchscreen
		pend = apTSC_READXY(&penx, &peny);
		//printf("PD:%d, PX:%03d, PY:%03d\n", pend, penx, peny);

		// Check for button press
		if (pend && (penx > 250) && (penx < 311))
		{
			if ((peny > 69) && (peny < 117))
				button = 1;
			if ((peny > 127) && (peny < 175))
				button = 2;
			if ((peny > 185) && (peny < 233))
				button = 3;
		}

		// Update PRESS every 500mS
		timer = (timer > 50) ? 0 : timer + 1;
		apSleep(10);
    } while (!button);

    return (button - 1);
}

int main (void)
{
    unsigned int x, y, button;
    unsigned short col;
    unsigned int id, fpgaid, apnote, rev, aid;

    // Enable AHB and APB clock
    CMSDK_SYSCON->AHBCLKCFG0SET = 0xF;  // GPIO
    CMSDK_SYSCON->APBCLKCFG0SET = 0x37;	// UART0, UART1, TIMER0, TIMER1, DUAL TIMER
    
    // UART init
    UartStdOutInit();

    // NVIC setup
    NVIC_ClearAllPendingIRQ();

    // Display the program version
    printf ("\n\n\nVersatile Express Cortex-M Prototyping System (V2M-MPS2) Demo\n");
    printf ("Version 1.1.1" " Build date: " __DATE__ "\n");
    printf ("Copyright (C) ARM Ltd 2013. All rights reserved.\n\n");

    rev = MPS2_SCC->CFG_REG4;
    
    printf("V2M-MPS2 revison %c\n\n", rev + 'A');
    
    fpgaid = MPS2_SCC->SCC_ID;
    aid = MPS2_SCC->SCC_AID;
    
    if (rev != EXTRACT_BITS(aid, 23, 20))
        printf("WARNING: Board revision does not match FPGA target revision\n\n");
    
    apnote = EXTRACT_BITS(fpgaid, 15, 4);
    
    printf("Application Note AN%x, FPGA build %d\n", apnote, EXTRACT_BITS(aid, 31, 24));

    // Display CPU ID
    id = SCB->CPUID;
	if(EXTRACT_BITS(id, 15, 8) == 0xC6)
		printf ("\nCPU: Cortex-M%d+ r%dp%d\n\n",EXTRACT_BITS(id, 7, 4),EXTRACT_BITS(id, 23, 20),EXTRACT_BITS(id, 3, 0));
	else
		printf ("\nCPU: Cortex-M%d r%dp%d\n\n",EXTRACT_BITS(id, 7, 4),EXTRACT_BITS(id, 23, 20),EXTRACT_BITS(id, 3, 0));

    // TSC init
    if(apTSC_INIT())
    {
        printf("\nERROR: Failed to initialise Touch Screen\n");
        
        printf("\nProgram terminated.\n");

        return TRUE;
    }

    // Display Colour bars on the VGA screen
    // VGA_Test ();

    // CLCD screen register setup
    GLCD_Initialize(); 

    // Run the demos
    do
    {
    	// Get main menu selection
    	button = mainmenu();

		// Mirror button press
        GLCD_SetWindowSize(0, 319, 64, 239);
        GLCD_Start();
        for(y = 64; y < 240; y++)
		{
            for(x = 0; x < 320; x++)
            {
            	col = flyerData[x + (y * 320)];
            	if ((x > 244) && (y > (64 + (button * 58))) && (y < (122 + (button * 58))) && (col == 0xFFFF))
    				GLCD_Write(Green);
            	else
    	    		GLCD_Write(col);
            }
		}
        GLCD_End();
    	apSleep(1000);

		// Run demos
		if (button == 0)
			apDEMO1_run();
		else if (button == 1)
			apDEMO2_run();
		else
			apDEMO3_run();

    } while (1);

}
